package com.cesgroup.bdc.attachment.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.cesgroup.bdc.attachment.dto.FileUploaderResponseDto;
import com.cesgroup.bdc.attachment.entity.Attachment;
import com.cesgroup.bdc.attachment.entity.DiskFile;
import com.cesgroup.bdc.attachment.entity.PrintNum;
import com.cesgroup.bdc.attachment.entity.RePrintApply;
import com.cesgroup.bdc.attachment.service.IAttachmentService;
import com.cesgroup.bdc.attachment.service.IDiskFileService;
import com.cesgroup.bdc.attachment.service.IPrintNumService;
import com.cesgroup.bdc.attachment.service.IRePrintApplyService;
import com.cesgroup.bdc.attachment.service.impl.PrintNumServiceImpl;
import com.cesgroup.bdc.logs.entity.Logs;
import com.cesgroup.bdc.message.entity.MsgType;
import com.cesgroup.bdc.message.service.INewMessagesService;
import com.cesgroup.bdc.util.CuiPage;
import com.cesgroup.bdc.util.PageVo;
import com.cesgroup.kingkong.core.security.model.IOrganization;
import com.cesgroup.kingkong.core.security.model.IUser;
import com.cesgroup.kingkong.core.security.service.AuthsystemService;
import com.cesgroup.kingkong.web.BaseServiceController;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 重打印申请表 前端控制器
 * </p>
 *
 * @author ChenKai
 * @since 2019-07-18
 */
@Controller
@RequestMapping("/re-print-apply")
public class RePrintApplyController extends BaseServiceController<RePrintApply, IRePrintApplyService> {

    @Autowired
    private AuthsystemService authsystemService;

    @Autowired
    private IPrintNumService printNumService;

    @Autowired
    private IAttachmentService attachmentService;

    @Autowired
    private IDiskFileService diskFileService;

    @Autowired
    private INewMessagesService messagesService;

    /**
     * @return org.springframework.web.servlet.ModelAndView
     * @Author ChenKai
     * @Description 重打印审批
     * @Date 2019-07-18 15:06
     */
    @RequestMapping("/responseIndex/{status}")
    public ModelAndView responseIndex(@PathVariable String status) {
        ModelAndView mv = new ModelAndView("rePrintApply/responseIndex");
        mv.addObject("status", status);
        return mv;
    }

    /**
     * @return org.springframework.web.servlet.ModelAndView
     * @Author ChenKai
     * @Description 重打印申请
     * @Date 2019-07-18 15:09
     */
    @RequestMapping("/requestIndex")
    public ModelAndView requestIndex() {
        ModelAndView mv = new ModelAndView("rePrintApply/requestIndex");
        return mv;
    }

    /**
     * @param pager
     * @return com.cesgroup.bdc.util.PageVo
     * @Author ChenKai
     * @Description 查看重打印审批列表
     * @Date 2019-07-18 15:06
     */
    @RequestMapping("/queryApplyViewList")
    @ResponseBody
    public PageVo queryApplyViewList(CuiPage<RePrintApply> pager, String status) {
        QueryWrapper<RePrintApply> queryWrapper = new QueryWrapper<>();
        IUser uu = getUser();
        IOrganization org = authsystemService.getOrganizationByUserId(uu.getId());
        if (status.equals("0")) {
            queryWrapper.eq("SEND_ATTACHMENT_DEPT_ID", org.getId()).eq("STATUS", "0");
        } else {
            if (!status.equals("all")) {
                queryWrapper.eq("SEND_ATTACHMENT_DEPT_ID", org.getId()).ne("STATUS", "0");
            }
        }

        IPage<RePrintApply> page = getBaseService().page(pager, queryWrapper);
        PageVo<RePrintApply> pv = PageVo.<RePrintApply>convertFromMybatisplusPage(page);
        return pv;
    }

    /**
     * @param pager
     * @return com.cesgroup.bdc.util.PageVo
     * @Author ChenKai
     * @Description 查看重打印申请列表
     * @Date 2019-07-18 15:12
     */
    @RequestMapping("/queryRequestViewList")
    @ResponseBody
    public PageVo queryRequestViewList(CuiPage<RePrintApply> pager) {
        QueryWrapper<RePrintApply> queryWrapper = new QueryWrapper<>();
        IUser uu = getUser();
        IOrganization org = authsystemService.getOrganizationByUserId(uu.getId());
        queryWrapper.eq("APPLY_DEPT_ID", org.getId()).eq("APPLY_USER_ID", uu.getId());
        IPage<RePrintApply> page = getBaseService().page(pager, queryWrapper);
        PageVo<RePrintApply> pv = PageVo.<RePrintApply>convertFromMybatisplusPage(page);
        return pv;
    }

    @RequestMapping("/answerApply/{id}/{reAttachmentId}/{type}")
    public ModelAndView answerApply(@PathVariable String id, @PathVariable String reAttachmentId,@PathVariable String type) {
        ModelAndView mv = new ModelAndView("rePrintApply/requestRePrint");
        mv.addObject("id", id);
        mv.addObject("reAttachmentId", reAttachmentId);
        mv.addObject("type", type);
        return mv;
    }


    @RequestMapping("/updateRePrintStatus")
    @ResponseBody
    public boolean updateRePrintStatus(RePrintApply rePrintApply, String id) {
        IUser uu = getUser();
        UpdateWrapper<RePrintApply> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("ID", id);
        rePrintApply.setApprovedUserName(uu.getUserName());
        rePrintApply.setApprovedUserId(uu.getId());
        rePrintApply.setApprovedDate(new Date());
        boolean flag = getBaseService().update(rePrintApply, updateWrapper);
        if (rePrintApply.getStatus().equals("1")) {
            RePrintApply rep = getBaseService().getById(id);
            UpdateWrapper<PrintNum> upPrintNum = new UpdateWrapper<>();
            upPrintNum.eq("ATTACH_ID", rep.getReAttachmentId()).eq("MIAL_RECEIVE_ID", rep.getMailReceiveId());
            PrintNum printNum = new PrintNum();
            printNum.setPrintNumb(rep.getRePrintNum());
            printNumService.update(printNum, upPrintNum);
        }

        messagesService.addNewMessage(getBaseService().getById(id).getApplyDeptId(),getBaseService().getById(id).getApplyUserId(),id, MsgType.REPRINT_RESULT_MSG);
        messagesService.updateMessageStatus(authsystemService.getOrganizationByUserId(this.getUser().getId()).getId(),id,MsgType.NEW_REPRINT_MSG);
        return flag;
    }


    @RequestMapping("/loadAttach/{reAttachmentId}")
    @ResponseBody
    public FileUploaderResponseDto find(@PathVariable String reAttachmentId) {
        Attachment attachment = attachmentService.getById(reAttachmentId);
        DiskFile diskFile = diskFileService.getById(attachment.getDiskFileId());
        FileUploaderResponseDto dto = new FileUploaderResponseDto(attachment, diskFile);
        return dto;
    }

    @RequestMapping("/queryByKeyWords")
    @ResponseBody
    public PageVo queryByKeyWords(CuiPage<RePrintApply> pager, RePrintApply rePrintApply, HttpServletRequest request) {
        String createTimeStart=request.getParameter("createTimeStart");
        String createTimeEnd=request.getParameter("createTimeEnd");
        String approvedDateStart=request.getParameter("approvedDateStart");
        String approvedDateEnd=request.getParameter("approvedDateEnd");
        QueryWrapper<RePrintApply> wrapper = Wrappers.query();

        if(StringUtils.isNotBlank(rePrintApply.getReAttachmentName())){//文件名
            wrapper.like("RE_ATTACHMENT_NAME",rePrintApply.getReAttachmentName());
        }
        if(StringUtils.isNotBlank(rePrintApply.getApplyUserName())){//申请人
            wrapper.like("APPLY_USER_NAME",rePrintApply.getApplyUserName());
        }
        if(StringUtils.isNotBlank(rePrintApply.getApprovedUserName())){//审批人
            wrapper.like("APPROVED_USER_NAME",rePrintApply.getApprovedUserName());
        }
        if(StringUtils.isNotBlank(createTimeStart)){//申请开始时间
            wrapper.ge("CREATE_TIME",createTimeStart);
        }
        if(StringUtils.isNotBlank(createTimeEnd)){//申请开始时间
            wrapper.le("CREATE_TIME",createTimeStart);
        }
        if(StringUtils.isNotBlank(approvedDateStart)){//申请开始时间
            wrapper.ge("APPROVED_DATE",approvedDateStart);
        }
        if(StringUtils.isNotBlank(approvedDateEnd)){//申请开始时间
            wrapper.le("APPROVED_DATE",approvedDateEnd);
        }
        wrapper.orderByDesc("CREATE_TIME");
        IPage<RePrintApply> p = getBaseService().page(pager, wrapper);
        PageVo<RePrintApply> pv = PageVo.convertFromMybatisplusPage(p);
        return pv;
    }


}
