package com.cesgroup.bdc.mail.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cesgroup.bdc.mail.entity.MailReceive;
import com.cesgroup.bdc.mail.entity.MailSend;
import com.cesgroup.bdc.mail.mapper.MailReceiveMapper;
import com.cesgroup.bdc.mail.service.IMailReceiveService;
import com.cesgroup.bdc.mail.service.IMailSendService;
import com.cesgroup.bdc.util.CuiPage;
import com.cesgroup.bdc.wdgl.vo.ConditionVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 收件信息 服务实现类
 * </p>
 *
 * @author shen.shaohua
 * @since 2019-07-08
 */
@Service
public class MailReceiveServiceImpl extends ServiceImpl<MailReceiveMapper, MailReceive> implements IMailReceiveService {
    @Autowired
    private IMailSendService mailSendService;

    /**
     * 根据状态查询
     *
     * @param sendId 发件id
     * @param status 状态
     * @return
     * @author: shen.shaohua
     * @since: 2019/7/11 18:04
     */
    public List<MailReceive> findBySendIdInStatus(String sendId, String... status) {
        QueryWrapper<MailReceive> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("mail_send_id", sendId).in("status", status);
        List<MailReceive> list = this.list(queryWrapper);
        return list;
    }

    /**
     * 根据发文id删除
     *
     * @param mailSendId
     * @return
     * @author: shen.shaohua
     * @since: 2019/7/11 23:51
     */
    public boolean delByMailSendId(String mailSendId) {
        Map<String, Object> columnMap = new HashMap<>();
        columnMap.put("mail_send_id", mailSendId);
        boolean b = this.removeByMap(columnMap);
        return b;
    }

    /**
     * 根据 ID 查询
     *
     * @param id
     * @return
     * @author: shen.shaohua
     * @since: 2019/7/16 10:32
     */
    public MailReceive selectById(String id) {
        MailReceive mailReceive = getBaseMapper().getById(id);
        return mailReceive;
    }

    /**
     * 归类统计
     *
     * @param groupColumn 统计字段
     * @param conditionVo 条件
     * @author: shen.shaohua
     * @since: 2019/7/23 17:17
     */
    public List<Map<String, Object>> statistics(String groupColumn, ConditionVo conditionVo) {
        conditionVo = this.parseConditionVo(conditionVo);
        List<Map<String, Object>> mapList = getBaseMapper().statistics(groupColumn, conditionVo);
        return mapList;
    }

    /**
     * 分页查询
     *
     * @param page        分页
     * @param conditionVo 查询条件
     * @return
     * @author: shen.shaohua
     * @since: 2019/7/16 10:31
     */
    public IPage<MailReceive> queryPage(IPage<MailReceive> page, ConditionVo conditionVo) {
        conditionVo = this.parseConditionVo(conditionVo);
        IPage<MailReceive> p = getBaseMapper().queryPage(page, conditionVo);
        return p;
    }

    /**
     * 查询
     *
     * @param conditionVo 查询条件
     * @return
     * @author: shen.shaohua
     * @since: 2019/7/16 15:43
     */
    public List<MailReceive> queryList(ConditionVo conditionVo) {
        conditionVo = this.parseConditionVo(conditionVo);
        List<MailReceive> p = getBaseMapper().queryList(conditionVo);
        return p;
    }

    /**
     * @param pager
     * @param mailReceive
     * @return com.baomidou.mybatisplus.core.metadata.IPage<com.cesgroup.bdc.mail.entity.MailReceive>
     * @Author ChenKai
     * @Description 根据条件查询收件列表
     * @Date 2019-07-16 10:28
     */
    @Override
    public IPage<MailReceive> queryReceiveListPage(CuiPage<MailReceive> pager, MailReceive mailReceive, String mailTitle, String sendTimeStart, String sendTimeEnd) {
        Date today = DateUtil.parseDate(DateUtil.formatDate(new Date()));
        try {
            Thread.sleep(1000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        IPage<MailReceive> p = getBaseMapper().queryReceiveListPage(pager, mailReceive, today, mailTitle, sendTimeStart, sendTimeEnd);
        return p;
    }

    /**
     * @param bussinessId
     * @return com.cesgroup.bdc.mail.entity.MailReceive
     * @Author ChenKai
     * @Description 根据收件id查新收件信息
     * @Date 2019-07-16 10:28
     */
    @Override
    public MailReceive queryReceiveFormById(String bussinessId) {
        return getBaseMapper().queryReceiveFormById(bussinessId);
    }

    private ConditionVo parseConditionVo(ConditionVo conditionVo) {
        if (StrUtil.isNotEmpty(conditionVo.getReceiveDept())) {
            String s = escaped(conditionVo.getReceiveDept());
            conditionVo.setReceiveDept(s);
        }
        if (StrUtil.isNotEmpty(conditionVo.getMailTitle())) {
            String s = escaped(conditionVo.getMailTitle());
            conditionVo.setMailTitle(s);
        }
        if (StrUtil.isNotEmpty(conditionVo.getFileKeyWord())) {
            String s = escaped(conditionVo.getFileKeyWord());
            conditionVo.setOriginalName(s);
        }
        return conditionVo;
    }

    private String escaped(String s) {
        s = s.replace("/", "//").replace("_", "/_")
                .replace("%", "/%");
        s = "%" + s + "%";
        return s;
    }

    /**
     * 收文全部签收时，发文状态改成完成
     *
     * @param mailSendId
     */
    public void updateMailSendStatus(String mailSendId) {
        QueryWrapper<MailReceive> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("MAIL_SEND_ID", mailSendId).eq("STATUS", "1");
        int count = this.count(queryWrapper);
        if (count == 0) {
            MailSend mailSend = mailSendService.getById(mailSendId);
            mailSend.setStatus("1");//1：完成
            mailSendService.updateById(mailSend);
        }
    }

    @Override
    public int querReceiveCounts(String deptId) {
        Date currentTime = new Date();
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        String dateString = formatter.format(currentTime);
        dateString+=" 23:59:59";
        return getBaseMapper().querReceiveCounts(deptId,dateString);
    }
}
