package com.cesgroup.bdc.print.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cesgroup.bdc.print.entity.PrintRecord;
import com.cesgroup.bdc.print.mapper.PrintRecordMapper;
import com.cesgroup.bdc.print.service.IPrintRecordService;
import com.cesgroup.bdc.print.vo.PrintShowVo;
import com.cesgroup.bdc.util.CuiPage;
import com.cesgroup.bdc.util.PrimaryKey;
import com.cesgroup.kingkong.core.security.model.IOrganization;
import com.cesgroup.kingkong.core.security.model.IUser;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 打印记录 服务实现类
 * </p>
 *
 * @author shen.shaohua
 * @since 2019-07-26
 */
@Service
public class PrintRecordServiceImpl extends ServiceImpl<PrintRecordMapper, PrintRecord> implements IPrintRecordService {
	/**
	 * 添加打印记录
	 *
	 * @param user
	 * @param org
	 * @param attachmentId
	 * @param mailReceiveId
	 * @author: shen.shaohua
	 * @since: 2019/7/26 17:30
	 */
	public void addOne(IUser user, IOrganization org, String attachmentId, String mailReceiveId) {
		PrintRecord printRecord = new PrintRecord();
		printRecord.setId(PrimaryKey.generateId());
		printRecord.setUserId(user.getId());
		printRecord.setUserName(user.getUserName());
		printRecord.setDeptId(org.getId());
		printRecord.setDeptName(org.getName());
		printRecord.setPrintTime(new Date());
		printRecord.setAttachmentId(attachmentId);
		printRecord.setMailReceiveId(mailReceiveId);
		this.save(printRecord);
	}

	public IPage<PrintShowVo> printStatistics(IPage page) {
		IPage<PrintShowVo> p = getBaseMapper().printStatistics(page);
		return p;
	}

	@Override
	public IPage<PrintShowVo> queryByKeyWords(IPage page, PrintShowVo printShowVo) {

		IPage<PrintShowVo> p = getBaseMapper().queryByKeyWords(page,parseExpireVo(printShowVo));
		return p;
	}

	@Override
	public List<PrintShowVo> getAll() {
		return getBaseMapper().getAll();
	}

	private PrintShowVo parseExpireVo(PrintShowVo printShowVo) {
		if (StrUtil.isNotEmpty(printShowVo.getDeptName())) {
			String s = escaped(printShowVo.getDeptName());
			printShowVo.setDeptName(s);
		}
		if (StrUtil.isNotEmpty(printShowVo.getFileName())) {
			String s = escaped(printShowVo.getFileName());
			printShowVo.setFileName(s);
		}
		if (StrUtil.isNotEmpty(printShowVo.getMailNo())) {
			String s = escaped(printShowVo.getMailNo());
			printShowVo.setMailNo(s);
		}

		return printShowVo;
	}


	private String escaped(String s) {
		s = s.replace("/", "//").replace("_", "/_")
				.replace("%", "/%");
		s = "%" + s + "%";
		return s;
	}
}
