package com.cesgroup.bdc.stamp.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cesgroup.authen4.ws.CoreHttpClientWS;
import com.cesgroup.authen4.ws.entity.User;
import com.cesgroup.bdc.attachment.controller.AttachmentController;
import com.cesgroup.bdc.attachment.entity.Attachment;
import com.cesgroup.bdc.attachment.entity.DiskFile;
import com.cesgroup.bdc.attachment.mapper.AttachmentMapper;
import com.cesgroup.bdc.attachment.mapper.DiskFileMapper;
import com.cesgroup.bdc.attachment.service.IAttachmentService;
import com.cesgroup.bdc.attachment.service.IDiskFileService;
import com.cesgroup.bdc.mail.entity.MailSend;
import com.cesgroup.bdc.mail.service.IMailSendService;
import com.cesgroup.bdc.stamp.constant.StampConstant;
import com.cesgroup.bdc.stamp.entity.HisStamp;
import com.cesgroup.bdc.stamp.entity.Stamp;
import com.cesgroup.bdc.stamp.entity.StampStatus;
import com.cesgroup.bdc.stamp.entity.UseSealHisInfo;
import com.cesgroup.bdc.stamp.mapper.HisStampMapper;
import com.cesgroup.bdc.stamp.mapper.StampMapper;
import com.cesgroup.bdc.stamp.mapper.StampStatusMapper;
import com.cesgroup.bdc.stamp.mapper.UseSealHisInfoMapper;
import com.cesgroup.bdc.stamp.service.IStampService;
import com.cesgroup.bdc.suwell.service.SuwellService;
import com.cesgroup.bdc.util.CompressUtil;
import com.cesgroup.bdc.util.PageVo;
import com.cesgroup.bdc.wfself.demo.StampApply;
import com.cesgroup.bdc.wfself.engine.constant.ProcConst;
import com.cesgroup.bdc.wfself.engine.mapper.ProcInstHisInfoMapper;
import com.cesgroup.bdc.wfself.engine.mapper.ProcInstRunVarMapper;
import com.cesgroup.bdc.wfself.engine.mapper.TaskRunInfoMapper;
import com.cesgroup.bdc.wfself.engine.mapper.TaskRunParticMapper;
import com.cesgroup.bdc.wfself.engine.service.IProcEngineService;
import com.cesgroup.bdc.wfself.engine.service.ITaskService;
import com.cesgroup.bdc.wfself.entity.ProcInstHisInfo;
import com.cesgroup.bdc.wfself.entity.ProcInstRunVar;
import com.cesgroup.bdc.wfself.entity.TaskRunInfo;
import com.cesgroup.bdc.wfself.entity.TaskRunPartic;
import com.cesgroup.kingkong.core.security.model.IOrganization;
import com.cesgroup.kingkong.core.security.model.IRole;
import com.cesgroup.kingkong.core.security.model.IUser;
import com.cesgroup.kingkong.core.security.service.AuthsystemService;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 发件信息 服务实现类
 * </p>
 *
 * @author shen.shaohua
 * @since 2019-07-08
 */
@Service
@Transactional
public class StampServiceImpl extends ServiceImpl<StampMapper, Stamp> implements IStampService {

    @Autowired
    private AuthsystemService authsystemService;
    @Autowired
    private AttachmentMapper attachmentMapper;
    @Autowired
    private IAttachmentService iAttachmentService;
    @Autowired
    private DiskFileMapper diskFileMapper;
    @Autowired
    private ITaskService iTaskService;
    @Autowired
    private IProcEngineService iProcEngineService;
    @Autowired
    private TaskRunInfoMapper taskRunInfoMapper;
    @Autowired
    private StampApply stampApply;
    @Autowired
    private ProcInstHisInfoMapper procInstHisInfoMapper;
    @Autowired
    private HisStampMapper hisStampMapper;
    @Autowired
    private StampStatusMapper stampStatusMapper;
    @Autowired
    private ProcInstRunVarMapper procInstRunVarMapper;
    @Autowired
    private TaskRunParticMapper taskRunParticMapper;
    @Autowired
    private SuwellService suwellService;
    @Autowired
    private AttachmentController attachmentController;

    @Autowired
    private UseSealHisInfoMapper useSealHisInfoMapper;

    @Autowired
    private IAttachmentService attachmentService;

    @Autowired
    private IDiskFileService diskFileService;

    @Autowired
    private IMailSendService mailSendService;

    @Value("${file.upload.path}")
    private String fileBasePath;


    @Override public Stamp selectOneStamp(String id) {
        Stamp stamp = getBaseMapper().selectById(id);
        return stamp;
    }

    /**
     * 提交
     * @param entity
     * @return
     * @throws Exception
     */
    @Override
    public Map<String, String> saveStamp(Stamp entity) throws Exception{
        IUser user = (IUser) SecurityUtils.getSubject().getPrincipal();
        IOrganization org = authsystemService.getOrganizationByUserId(user.getId());
        //流程定义id，申请人id，下个节点申请人id，操作人id，作为参数
        String procDefId = stampApply.getProcDefId();
        String userId = user.getId();
        String receiveDept = entity.getYzDept();
        String currOrgId = org.getId();

        Map<String, String> procParam=new HashMap<>();
        procParam.put(ProcConst.PARTIC_ASSIGNEE_LIST, userId);
        //调用启动程序返回实例id
        String procInstId = iProcEngineService.startProcess(procDefId, userId, procParam);

        //保存stamp用章信息表
        entity.setSqTime(new Date());
        entity.setProcinstid(procInstId);
        entity.setApplyPersonId(user.getId());
        entity.setApplyPersonName(user.getUserName());
        entity.setApplyDeptId(org.getId());
        entity.setApplyDeptName(org.getName());
        if (entity.getType().equals("2")) {
            JSONArray jsonArray = new JSONArray(receiveDept);
            StringBuilder stringBuilder = new StringBuilder();
            for (int i = 0; i < jsonArray.size(); i++) {
                stringBuilder.append(",").append(((JSONObject)jsonArray.get(i)).get("deptId"));
            }
            entity.setWdwDeptId(stringBuilder.substring(1));
        }

        entity.setStatus(StampConstant.STATUS_TO_DEAL_TEXT);
        getBaseMapper().insert(entity);

        //根据实例id查询运行表获取任务id
        QueryWrapper<TaskRunInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("PROC_INST_ID", procInstId);
        List<TaskRunInfo> taskRunInfoList = taskRunInfoMapper.selectList(queryWrapper);
        // 申请人处理
        jsyApprove(taskRunInfoList.get(0).getId(), entity.getId(), null);
        Map<String, String> mm = new HashMap<>();
        mm.put("status", "success");
        return mm;
    }


    @Override
    public PageVo stampQueryApplyList(String sendTimeStart, String sendTimeEnd, String mailTitle, String status, String type, IPage<HisStamp> pager) {
        //根据受托人查询历史表数据
        IUser user = (IUser) SecurityUtils.getSubject().getPrincipal();
        QueryWrapper<HisStamp> queryWrapper = new QueryWrapper<>();
        if (StringUtils.isNotBlank(type)&&type.equals("1")) {
            queryWrapper.eq("APPLY_PERSON_ID", user.getId());
            queryWrapper.and(e->e.isNull("P_STATUS").or().eq("NODE_CODE","end"));
        }else {
            queryWrapper.eq("ASSGINEE_ID", user.getId());
            queryWrapper.ne("NODE_CODE","start");
            queryWrapper.ne("NODE_CODE","end");
        }
        if (StringUtils.isNotBlank(mailTitle)) {
            queryWrapper.like("MAIL_TITLE", mailTitle);
        }
        if (StringUtils.isNotBlank(status)) {
            //1:未完成0：完成
            if (status.equals("1")) {
                queryWrapper.eq("P_STATUS", "completed");
            }else if(status.equals("0")){
                queryWrapper.isNull("P_STATUS");
            }else if(status.equals("2")){
                queryWrapper.eq("S_STATUS", StampConstant.STATUS_TO_DEAL_TEXT);
            }else if(status.equals("3")){
                queryWrapper.eq("S_STATUS", StampConstant.STATUS_FROM_BACK_TEXT);
            }else if(status.equals("4")){
                queryWrapper.eq("S_STATUS", StampConstant.STATUS_TO_APPROVE_TEXT);
            }else if(status.equals("5")){
                queryWrapper.eq("S_STATUS", StampConstant.STATUS_TO_IMPRINT_TEXT);
            }else if(status.equals("6")){
                queryWrapper.eq("S_STATUS", StampConstant.STATUS_COMPLETED_TEXT);
            }else if(status.equals("7")){
                queryWrapper.ne("S_STATUS", StampConstant.STATUS_COMPLETED_TEXT);
            }else if(status.equals("8")){
                queryWrapper.eq("S_STATUS", StampConstant.STATUS_SIGN_TEXT);
            }

        }
        if (StringUtils.isNotBlank(sendTimeStart)) {
            Date startDate = DateUtil.parseDate(sendTimeStart);
            queryWrapper.ge("SQ_TIME", startDate);
        }
        if (StringUtils.isNotBlank(sendTimeEnd)) {
            Date endDate = DateUtil.parseDate(sendTimeEnd);
            endDate = DateUtil.offsetDay(endDate, 1);
            queryWrapper.lt("SQ_TIME", endDate);
        }
        queryWrapper.orderByAsc("P_STATUS").orderByAsc("START_TIME");
        IPage<HisStamp> p = hisStampMapper.selectPage(pager, queryWrapper);
        PageVo<HisStamp> pv = PageVo.<HisStamp>convertFromMybatisplusPage(p);
        return pv;
    }

    @Override public HisStamp selectByExecId(String execId) {
        QueryWrapper<HisStamp> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("EXEC_ID", execId);
        HisStamp hisStamp = hisStampMapper.selectOne(queryWrapper);
        return hisStamp;
    }


    @Override
    public void approve(String execId, String reason, Stamp stamp) throws Exception{

        //获取当前用户
        IUser user = (IUser) SecurityUtils.getSubject().getPrincipal();
        //获取当前用户组织
        IOrganization org = authsystemService.getOrganizationByUserId(user.getId());

        //根据实例id查询运行表数据
        QueryWrapper<TaskRunInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("PROC_INST_ID", stamp.getProcinstid());
        List<TaskRunInfo> taskRunInfoList = taskRunInfoMapper.selectList(queryWrapper);
        //任务节点code
        String taskCode = taskRunInfoList.get(0).getTaskCode();

        String stampId = stamp.getId();

        // 申请人接待您，流程开始不会进入，只有退回后处理才会进入
        if(taskCode.equals("apply")){
            jsyApprove(execId, stamp.getId(), stamp);
        } else if (taskCode.equals("bdwldsp")) {//本单位领导
            dwldApprove(execId, stampId, reason, StampConstant.STATUS_BDWLDSP);
        }else if (taskCode.equals("bdwgzgly")) {//本单位盖章
            gzglyApprove(execId, stampId, StampConstant.STATUS_BDWGZGLY);
        }else if (taskCode.equals("wdwjsy")) {//外单位接收员
            jsyApprove(execId, stampId, null);
        }else if (taskCode.equals("wdwldsp")) {//外单位领导
            dwldApprove(execId, stampId, reason, StampConstant.STATUS_WDWLDSP);
        }else if (taskCode.equals("wdwgzgly")) {//外单位盖章
            gzglyApprove(execId, stampId, StampConstant.STATUS_WDWGZGLY);
        }else if (taskCode.equals("dqr")){
            signApprove(execId,stampId);
        }
    }

    /**
     * 本单位领导审批处理
     * @param execId
     * @param stampId
     * @param reason
     * @param nodeFrom
     * @throws Exception
     */
    public void dwldApprove(String execId, String stampId, String reason, String nodeFrom) throws Exception{

        //获取当前用户
        IUser user = (IUser) SecurityUtils.getSubject().getPrincipal();
        //获取当前用户组织
        IOrganization org = authsystemService.getOrganizationByUserId(user.getId());

        //根据id查询用章信息
        QueryWrapper<Stamp> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("ID", stampId);
        Stamp stamp = getBaseMapper().selectOne(queryWrapper1);

        // 退回
        if(StringUtils.isNotBlank(reason)){

            Map<String, String> paramMap = new HashMap<>();
            // 退回流向值
            paramMap.put("path", "0");
            paramMap.put(ProcConst.PARTIC_ASSIGNEE_LIST, stamp.getApplyPersonId());
            iTaskService.complete(execId, user.getId(), paramMap);

            StampStatus stampStatus = new StampStatus();
            stampStatus.setStatus("退回");
            stampStatus.setExecId(execId);
            stampStatus.setReason(reason);
            stampStatus.setStampId(stampId);
            stampStatus.setProcInstId(stamp.getProcinstid());

            //保存业务数据
            stampStatusMapper.insert(stampStatus);
            //重新生成ofd文件
            updateFile(stamp.getId());
            stamp.setStatus(StampConstant.STATUS_FROM_BACK_TEXT);
        } else {

            /* 判断是本单位领导接单处理，还是外单位领导处理*/

            // 本单位领导
            if(StampConstant.STATUS_BDWLDSP.equals(nodeFrom)){
                // 有值，流向外单位分支处理
                if(StringUtils.isNotBlank(stamp.getWdwDeptId())){

                    String wdwDeptId = stamp.getWdwDeptId();
                    List<String> wdwDeptIdList = new ArrayList<>();
                    Collections.addAll(wdwDeptIdList, wdwDeptId.split(","));
                    wdwDeptId = wdwDeptIdList.remove(0);

                    // 获取接收人，需要修改角色code
                    String nextTaskUserId = getUserId(StampConstant.AUTH_ROLE_JSY, wdwDeptId);

                    Map<String, String> paramMap = new HashMap<>();
                    paramMap.put("path", "2");
                    paramMap.put(ProcConst.PARTIC_ASSIGNEE_LIST, nextTaskUserId);
                    iTaskService.complete(execId, user.getId(), paramMap);
                    stamp.setStatus(StampConstant.STATUS_TO_DEAL_TEXT);

                    // 取出一个外单位id后，以原有的格式存入
                    if(wdwDeptIdList.size() != 0){
                        stamp.setWdwDeptId(wdwDeptIdList.stream().collect(Collectors.joining(",")));
                    }else{
                        stamp.setWdwDeptId("");
                    }

                    // 流向本单位盖章
                }else{
                    // 获取盖章员id，需要修改角色code
                    String nextTaskUserId = getUserId(StampConstant.AUTH_ROLE_GZY, org.getId());
                    Map<String, String> paramMap = new HashMap<>();
                    paramMap.put("path", "1");
                    paramMap.put(ProcConst.PARTIC_ASSIGNEE_LIST, nextTaskUserId);
                    iTaskService.complete(execId, user.getId(), paramMap);
                    stamp.setStatus(StampConstant.STATUS_TO_IMPRINT_TEXT);
                }

             // 外单位领导处理
            } else{

                // 获取盖章员id，需要修改角色code
                String nextTaskUserId = getUserId(StampConstant.AUTH_ROLE_GZY, org.getId());
                Map<String, String> paramMap = new HashMap<>();
                paramMap.put(ProcConst.PARTIC_ASSIGNEE_LIST, nextTaskUserId);
                paramMap.put("path", "1");
                iTaskService.complete(execId, user.getId(), paramMap);
                stamp.setStatus(StampConstant.STATUS_TO_IMPRINT_TEXT);
            }
        }
        // 更新业务数据
        getBaseMapper().updateById(stamp);
    }

    /**
     * 盖章处理
     * @param execId
     * @param stampId
     * @param nodeFrom
     */
    public void gzglyApprove(String execId, String stampId, String nodeFrom) throws Exception{
        //获取当前用户
        IUser user = (IUser) SecurityUtils.getSubject().getPrincipal();
        //获取当前用户组织
        IOrganization org = authsystemService.getOrganizationByUserId(user.getId());

        //根据id查询用章信息
        QueryWrapper<Stamp> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("ID", stampId);
        Stamp stamp = getBaseMapper().selectOne(queryWrapper1);


        /* 判断是本单位盖章处理，还是外单位盖章处理*/

        // 本单位盖章
        if(StampConstant.STATUS_BDWGZGLY.equals(nodeFrom)){
            Map<String, String> paramMap = new HashMap<>();
            paramMap.put(ProcConst.PARTIC_ASSIGNEE_LIST, stamp.getApplyPersonId());
            iTaskService.complete(execId, user.getId(), paramMap);
            stamp.setStatus(StampConstant.STATUS_SIGN_TEXT);

         // 外单位盖章
        } else {

            String wdwDeptId = stamp.getWdwDeptId();
            // 有未完成盖章的单位
            if(StringUtils.isNotBlank(wdwDeptId)){

                List<String> wdwDeptIdList = new ArrayList<>();
                Collections.addAll(wdwDeptIdList, wdwDeptId.split(","));
                wdwDeptId = wdwDeptIdList.remove(0);

                // 获取接收人，需要修改角色code
                String nextTaskUserId = getUserId(StampConstant.AUTH_ROLE_JSY, wdwDeptId);

                Map<String, String> paramMap = new HashMap<>();
                paramMap.put("path", "1");
                paramMap.put(ProcConst.PARTIC_ASSIGNEE_LIST, nextTaskUserId);
                iTaskService.complete(execId, user.getId(), paramMap);
                stamp.setStatus(StampConstant.STATUS_TO_DEAL_TEXT);

                // 取出一个外单位id后，以原有的格式存入
                if(wdwDeptIdList.size() != 0){
                    stamp.setWdwDeptId(wdwDeptIdList.stream().collect(Collectors.joining(",")));
                } else {
                    stamp.setWdwDeptId("");
                }

            } else {
                Map<String, String> paramMap = new HashMap<>();
                paramMap.put("path", "0");
                paramMap.put(ProcConst.PARTIC_ASSIGNEE_LIST, stamp.getApplyPersonId());
                iTaskService.complete(execId, user.getId(), paramMap);
                stamp.setStatus(StampConstant.STATUS_SIGN_TEXT);
            }
        }

        // 更新业务数据
        getBaseMapper().updateById(stamp);
    }

    /**
     * 签收
     * @param execId
     * @param stampId
     */
    public void signApprove(String execId, String stampId) throws Exception{
        //获取当前用户
        IUser user = (IUser) SecurityUtils.getSubject().getPrincipal();
        //获取当前用户组织
        IOrganization org = authsystemService.getOrganizationByUserId(user.getId());

        //根据id查询用章信息
        QueryWrapper<Stamp> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("ID", stampId);
        Stamp stamp = getBaseMapper().selectOne(queryWrapper1);

        Map<String, String> paramMap = new HashMap<>();
        paramMap.put(ProcConst.PARTIC_ASSIGNEE_LIST, user.getId());
        iTaskService.complete(execId, user.getId(), paramMap);
        stamp.setStatus(StampConstant.STATUS_COMPLETED_TEXT);
        // 更新业务数据
        getBaseMapper().updateById(stamp);
    }


    /**
     * 接收人（申请人）
     * @param execId
     * @param stampId
     */
    public void jsyApprove(String execId, String stampId, Stamp stamp) throws Exception{
        //获取当前用户
        IUser user = (IUser) SecurityUtils.getSubject().getPrincipal();
        //获取当前用户组织
        IOrganization org = authsystemService.getOrganizationByUserId(user.getId());

        if(stamp == null){
            //根据id查询用章信息
            QueryWrapper<Stamp> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.eq("ID", stampId);
            stamp = getBaseMapper().selectOne(queryWrapper1);
        }else {
            if (stamp.getType().equals("2")) {
                JSONArray jsonArray = new JSONArray(stamp.getYzDept());
                StringBuilder stringBuilder = new StringBuilder();
                for (int i = 0; i < jsonArray.size(); i++) {
                    stringBuilder.append(",").append(((JSONObject)jsonArray.get(i)).get("deptId"));
                }
                stamp.setWdwDeptId(stringBuilder.substring(1));
            }
        }

        // 获取领导，需要修改角色code
        String nextTaskUserId = getUserId(StampConstant.AUTH_ROLE_DWLD, org.getId());

        Map<String, String> paramMap = new HashMap<>();
        paramMap.put(ProcConst.PARTIC_ASSIGNEE_LIST, nextTaskUserId);
        paramMap.put("stampId", stampId);
        iTaskService.complete(execId, user.getId(), paramMap);
        stamp.setStatus(StampConstant.STATUS_TO_APPROVE_TEXT);
        // 更新业务数据
        getBaseMapper().updateById(stamp);
    }

    @Override
    public HisStamp selectByhpIdDwld(String hpId) {
        QueryWrapper<HisStamp> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("PRE_NODE_ID", hpId);
        queryWrapper.eq("NODE_CODE", "apply");
        HisStamp hisStamp = hisStampMapper.selectOne(queryWrapper);
        return hisStamp;
    }

    @Override
    public HisStamp selectByhpIdApply(String preNodeId) {
        QueryWrapper<HisStamp> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("P_ID", preNodeId);
        HisStamp hisStamp = hisStampMapper.selectOne(queryWrapper);
        return hisStamp;
    }

    @Override public List<Map<String, Object>> getReason(String procInstId) {
        List<Map<String, Object>> list=new ArrayList<>();

        QueryWrapper<StampStatus> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("PROC_INST_ID", procInstId);
        List<StampStatus> stampStatuses = stampStatusMapper.selectList(queryWrapper);
        for (int i = 0; i < stampStatuses.size(); i++) {
            Map<String, Object> map=new HashMap<>();
            //根据执行id查询历史表
            QueryWrapper<ProcInstHisInfo> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.eq("EXEC_ID", stampStatuses.get(i).getExecId());
            ProcInstHisInfo procInstHisInfo = procInstHisInfoMapper.selectOne(queryWrapper1);
            map.put("handlerName",procInstHisInfo.getHandlerName());
            map.put("endTime",procInstHisInfo.getEndTime());
            map.put("reason",stampStatuses.get(i).getReason());
            list.add(map);
        }
        return list;
    }

    @Override
    public Integer getWdCount() {
        //根据申请人查询历史表数据
        IUser user = (IUser) SecurityUtils.getSubject().getPrincipal();
        QueryWrapper<HisStamp> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("APPLY_PERSON_ID", user.getId());
        queryWrapper.isNull("P_STATUS");
        Integer integer = hisStampMapper.selectCount(queryWrapper);
        return integer;
    }

    @Override
    public Integer getYzCount() {
        //根据申请人查询历史表数据
        IUser user = (IUser) SecurityUtils.getSubject().getPrincipal();
        QueryWrapper<HisStamp> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("ASSGINEE_ID", user.getId());
        queryWrapper.isNull("P_STATUS");
        Integer integer = hisStampMapper.selectCount(queryWrapper);
        return integer;
    }

    @Override
    public void deleteStampAndPHis(String stampId) {
        Stamp stamp = getBaseMapper().selectById(stampId);
        String procinstid = stamp.getProcinstid();
        //根据实例id删除工作流中所有数据,删除用章表和用章退回信息表
        QueryWrapper<TaskRunInfo> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("PROC_INST_ID", procinstid);
        QueryWrapper<ProcInstHisInfo> queryWrapper2 = new QueryWrapper<>();
        queryWrapper2.eq("PROC_INST_ID", procinstid);
        QueryWrapper<ProcInstRunVar> queryWrapper3 = new QueryWrapper<>();
        queryWrapper3.eq("PROC_INST_ID", procinstid);
        QueryWrapper<TaskRunPartic> queryWrapper4 = new QueryWrapper<>();
        queryWrapper4.eq("PROC_INST_ID", procinstid);
        QueryWrapper<StampStatus> queryWrapper5 = new QueryWrapper<>();
        queryWrapper5.eq("PROC_INST_ID", procinstid);
        taskRunInfoMapper.delete(queryWrapper1);
        procInstHisInfoMapper.delete(queryWrapper2);
        procInstRunVarMapper.delete(queryWrapper3);
        taskRunParticMapper.delete(queryWrapper4);
        stampStatusMapper.delete(queryWrapper5);
        getBaseMapper().deleteById(stampId);
    }

    @Override
    public void addUseSealHisInfo(UseSealHisInfo useSealHisInfo) {
        useSealHisInfoMapper.insert(useSealHisInfo);
    }

    @Override
    public void download(HttpServletRequest request, HttpServletResponse response) {
        String attachmentId = request.getParameter("attachmentId");
        //如果传入的是附件id
        if (attachmentId != null) {
            //根据附件id获取文件id
            Attachment attachment = attachmentService.getById(attachmentId);
            //文件id
            String diskFileId = attachment.getDiskFileId();
            //根据文件id获取文件信息
            DiskFile diskFile = diskFileService.getById(attachment.getDiskFileId());
            //根据附件关联id获取发文信息
            MailSend mailSendbyId = mailSendService.getById(attachment.getRelevanceId());
            //根据文件id获取盖章历史信息
            QueryWrapper<UseSealHisInfo> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("FILE_ID",diskFileId);
            List<UseSealHisInfo> useSealHisInfos = useSealHisInfoMapper.selectList(queryWrapper);
            if (useSealHisInfos.size()==0) {
                attachmentController.downloadOriginal(attachmentId,response,request);
            }else {
                List<Map<String, String>> filePaths = new ArrayList<>();
                Map<String, String> map=new HashMap<>();
                map.put("filePath",fileBasePath+diskFile.getPath());
                map.put("fileName",diskFile.getOriginalName());
                Map<String, String> map1=new HashMap<>();
                map1.put("filePath",fileBasePath+diskFile.getPath().substring(0,diskFile.getPath().lastIndexOf("."))+".ofd");
                map1.put("fileName",diskFile.getOriginalName().substring(0,diskFile.getOriginalName().lastIndexOf("."))+".ofd");
                filePaths.add(map);
                filePaths.add(map1);
                String zipName = diskFile.getOriginalName().substring(0,diskFile.getOriginalName().lastIndexOf(".")) + ((int) (Math.random() * 10000)) + ".zip";
                String zipPath = fileBasePath+diskFile.getPath().substring(0,6) + "/" + zipName;
                CompressUtil.compress(filePaths, zipPath, false);
                File pocZipFile = new File(zipPath);
                CompressUtil.downloadZip(response, zipName, pocZipFile, request);
                pocZipFile.delete();
            }
        }
    }

    /**
     * 获取指定组织下，存在指定角色的用户
     * @param roleCode
     * @param orgId
     * @return
     */
    private String getUserId(String roleCode, String orgId){
        //查询角色下所有用户信息
        List<User> userList = CoreHttpClientWS.getCoreHttpClientWS().queryUsersByOrgId(orgId);

        //筛选本单位审批领导
        for (User user : userList) {
            List<IRole> roleList = authsystemService.findRolesByUserId(user.getId());
            for(IRole role : roleList){
                if(roleCode.equals(role.getCode())){
                    return user.getId();
                }
            }
        }
        return null;
    }
    /**
     * 根据stampId修改文件
     * @param stampId
     * @return
     */
    private String updateFile(String stampId){
        //根据stampId查询发文id
        QueryWrapper<Stamp> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("ID", stampId);
        Stamp stamp = getBaseMapper().selectOne(queryWrapper);
        String mailSendId = stamp.getMailSendId();
        //根据发文id查询文件关联id
        QueryWrapper<Attachment> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("RELEVANCE_ID", mailSendId);
        List<Attachment> attachments = attachmentMapper.selectList(queryWrapper1);
        for (int i = 0; i < attachments.size(); i++) {
            //根据关联id查询文件路径和文件名
            File oldFile = iAttachmentService.getFile(attachments.get(i));//原文件
            String oldFileName = oldFile.getName();
            String newFileName = oldFileName.substring(0, oldFileName.lastIndexOf(".")) + ".ofd";
            //删除原ofd文件，重新生成ofd文件
            File file=new File(oldFile.getParent(),newFileName);
            file.delete();
            new Thread(() -> {
                try {
                    suwellService.officeToOFD(oldFile.getParent()+"/"+oldFileName, oldFile.getParent()+"/"+newFileName);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }).start();
        }
        return null;
    }
}
