package com.cesgroup.bdc.suwell.service;

import com.suwell.ofd.custom.agent.HTTPAgent;
import com.suwell.ofd.custom.wrapper.model.MarkPosition;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;

/**
 * <p>
 * 数科ofd服务类
 * </p>
 *
 * @author: shen.shaohua
 * @since: 2019/9/6 13:51
 */
@Service
public class SuwellService {
  private Log logger = LogFactory.getLog(getClass());

  /**
   * 数科ofd服务url
   */
  @Value("${suwell.ofd.HTTPAgent.url}")
  private String suwellofdHTTPAgentUrl;

  private HTTPAgent httpAgent = null;

  private HTTPAgent getHttpAgent() {
    if (httpAgent == null) {
      httpAgent = new HTTPAgent(suwellofdHTTPAgentUrl);
    }
    return httpAgent;
  }

  /**
   * 将单个办公文件（Office文件如doc等、版式文件如pdf、xps等）转换为OFD文件
   *
   * @param sourcePath 源文件路径
   * @param targetPath 生成的OFD文件路径（后缀名.ofd）
   * @author: shen.shaohua
   * @since: 2019/9/6 13:50
   */
  public boolean officeToOFD(String sourcePath, String targetPath) {
    boolean b = false;
    try {
      logger.info("ofd转换：" + sourcePath + " --> " + targetPath);
      HTTPAgent ha = getHttpAgent();
      ha.officeToOFD(new File(sourcePath), new FileOutputStream(targetPath));
      b = true;
      logger.info("ofd转换成功");
    } catch (Exception e) {
      logger.error("ofd转换失败：" + e.getMessage());
      e.printStackTrace();
    }
    return b;
  }

  /**
   * 将单个办公文件（Office文件如doc等、版式文件如pdf、xps等）转换为PDF文件
   *
   * @param sourcePath 源文件路径
   * @param targetPath 生成的PDF文件路径（后缀名.pdf）
   * @author: shen.shaohua
   * @since: 2019/9/9 18:34
   */
  public boolean OFDToPDF(String sourcePath, String targetPath) {
    boolean b = false;
    try {
      logger.info("pdf转换：" + sourcePath + " --> " + targetPath);
      HTTPAgent ha = getHttpAgent();
      ha.OFDToPDF(new File(sourcePath), new FileOutputStream(targetPath));
      b = true;
      logger.info("pdf转换成功");
    } catch (Exception e) {
      logger.error("pdf转换失败：" + e.getMessage());
      e.printStackTrace();
    }
    return b;
  }

  /**
   * 第一页添加图片水印
   *
   * @param srcFilePath    原ofd/word文件路径
   * @param targetFilePath 生成的ofd文件路径
   * @param imgPath        水印图片路径
   * @param x              x坐标，单位毫米
   * @param y              y坐标，单位毫米
   * @author: shen.shaohua
   * @since: 2019/9/6 17:43
   */
  public boolean addImageMark(String srcFilePath, String targetFilePath, String imgPath,
                              float x, float y) {
    boolean b = addImageMark(srcFilePath, targetFilePath, imgPath, x, y, new int[]{1});
    return b;
  }

  /**
   * 添加图片水印
   *
   * @param srcFilePath    原ofd/word文件路径
   * @param targetFilePath 生成的ofd文件路径
   * @param imgPath        水印图片路径
   * @param x              x坐标，单位毫米
   * @param y              y坐标，单位毫米
   * @param pageindex      填加水印的页码集合（从1开始）
   * @author: shen.shaohua
   * @since: 2019/9/6 14:04
   */
  public boolean addImageMark(String srcFilePath, String targetFilePath, String imgPath,
                              float x, float y, int[] pageindex) {
    boolean b = false;
    try {
      logger.info("ofd添加图片水印：" + srcFilePath + " --> " + targetFilePath + "###水印图片：" + imgPath);
      HTTPAgent ha = getHttpAgent();
      File srcFile = new File(srcFilePath);
      FileOutputStream targetFileOut = new FileOutputStream(targetFilePath);
      InputStream imge = new FileInputStream(imgPath);
      String imageType = imgPath.substring(imgPath.lastIndexOf('.'));//图片后缀名
      /*
      MarkPosition(float x, float y, float width, float height, int[] index)
      参数1：x        x坐标，单位毫米
      参数2：y        y坐标，单位毫米
      参数3：width	   水印的宽度，单位毫米
      参数4：height  水印的高度，单位毫米
      参数5：index   填加水印的页码集合，从1开始，所有页使用常量MarkPosition.INDEX_ALL
      */
      //A4纸规格尺寸：210mm×297mm
      MarkPosition mk = new MarkPosition(x, y, 210F, 297F, pageindex);
      ha.addImageMark(srcFile, targetFileOut, imge, imageType, mk, true, true);
      b = true;
      logger.info("ofd添加图片水印成功");
    } catch (Exception e) {
      logger.error("ofd添加图片水印失败：" + e.toString());
      e.printStackTrace();
    }
    return b;
  }
}
