package com.cesgroup.bdc.wfself.demo;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.cesgroup.bdc.wfself.node.event.EndEvent;
import com.cesgroup.bdc.wfself.node.event.StartEvent;
import com.cesgroup.bdc.wfself.node.gateway.ExcluGateway;
import com.cesgroup.bdc.wfself.node.gateway.ParaGateway;
import com.cesgroup.bdc.wfself.node.task.UserTask;
import com.cesgroup.bdc.wfself.def.CommonNode;
import com.cesgroup.bdc.wfself.def.CommonProcDef;
import com.cesgroup.bdc.wfself.def.INode;
import com.cesgroup.bdc.wfself.def.ITask;
import org.springframework.stereotype.Component;

public class StampApply extends CommonProcDef {

    // 开始节点
    private final StartEvent startEvent = new StartEvent("start", "开始节点");

    // 填写申请单
    private final UserTask apply = new UserTask("apply", "申请人");

    // 本单位领导审批
    private final UserTask bdwldsp = new UserTask("bdwldsp", "本单位领导审批");

    // 本单位管理员盖章
    private final UserTask bdwgzgly = new UserTask("bdwgzgly", "本单位盖章管理员");

    // 外单位管理员接收
    private final UserTask wdwjsy = new UserTask("wdwjsy", "外单位接收员");

    // 外单位领导审批
    private final UserTask wdwldsp = new UserTask("wdwldsp", "外单位领导审批");

    // 外单位管理员盖章
    private final UserTask wdwgzgly = new UserTask("wdwgzgly", "外单位盖章管理员");

    // 签收节点
    private final UserTask dqr = new UserTask("dqr", "待确认");

    // 结束节点
    private final EndEvent endEvent = new EndEvent("end", "结束节点");

    /**
     * 第一个排他网关
     */
    private final ExcluGateway excluGateway1 = new ExcluGateway("excluGateway1","") {
        @Override
        public INode condition(String param, List<INode> nodeList) {
            // param，0代表退回之申请人、1代表本单位盖章、2代表外单位接收
            return nodeList.get(Integer.valueOf(param));
        }
    };
    /**
     * 第二个排他网关
     */
    private final ExcluGateway excluGateway2 = new ExcluGateway("excluGateway2","") {
        @Override
        public INode condition(String param, List<INode> nodeList) {
            //  param，0代表退回之申请人、1代表本单位盖章
            return nodeList.get(Integer.valueOf(param));
        }
    };
    /**
     * 第三个排他网关
     */
    private final ExcluGateway excluGateway3 = new ExcluGateway("excluGateway3","") {
        @Override
        public INode condition(String param, List<INode> nodeList) {
            //  param，0代表结束、1代表存在外单位盖章流程
            return nodeList.get(Integer.valueOf(param));
        }
    };

    @Override
    public StartEvent getStartEvent() {
        return this.startEvent;
    }

    public StampApply(){
        this.setName("用章处理");
        this.setProcDefCode("stampApply");
        this.setVersion("1.0");
        this.setProcDefId(this.getProcDefCode() + ":" + this.getVersion());
    }

    {
        
        /*=================== 设置节点对象 ==================*/
        Map<String, CommonNode> procDefNodeMap = this.getProcDefNodeMap();
        procDefNodeMap.put(startEvent.getCode(), startEvent);
        procDefNodeMap.put(apply.getCode(), apply);
        procDefNodeMap.put(bdwldsp.getCode(), bdwldsp);
        procDefNodeMap.put(bdwgzgly.getCode(), bdwgzgly);
        procDefNodeMap.put(wdwjsy.getCode(), wdwjsy);
        procDefNodeMap.put(wdwldsp.getCode(), wdwldsp);
        procDefNodeMap.put(wdwgzgly.getCode(), wdwgzgly);
        procDefNodeMap.put(excluGateway1.getCode(), excluGateway1);
        procDefNodeMap.put(excluGateway2.getCode(), excluGateway2);
        procDefNodeMap.put(excluGateway3.getCode(), excluGateway3);
        procDefNodeMap.put(dqr.getCode(), dqr);
        procDefNodeMap.put(endEvent.getCode(), endEvent);
        
        // 存储流程定义信息
        Map<String, Object> procDefMap = this.getProcDefMap();

        /*=================== 定义流程信息 ==================*/

        procDefMap.put(startEvent.getCode(), apply);
        procDefMap.put(apply.getCode(), bdwldsp);
        procDefMap.put(bdwldsp.getCode(), excluGateway1);
        List<INode> excluGateway1Node = new ArrayList<>();
        excluGateway1Node.add(apply);
        excluGateway1Node.add(bdwgzgly);
        excluGateway1Node.add(wdwjsy);
        procDefMap.put(excluGateway1.getCode(), excluGateway1Node);
        procDefMap.put(bdwgzgly.getCode(), dqr);
        procDefMap.put(wdwjsy.getCode(), wdwldsp);
        procDefMap.put(wdwldsp.getCode(), excluGateway2);
        List<INode> excluGateway2Node = new ArrayList<>();
        excluGateway2Node.add(apply);
        excluGateway2Node.add(wdwgzgly);
        procDefMap.put(excluGateway2.getCode(), excluGateway2Node);
        procDefMap.put(wdwgzgly.getCode(), excluGateway3);
        List<INode> excluGateway3Node = new ArrayList<>();
        excluGateway3Node.add(dqr);
        excluGateway3Node.add(wdwjsy);
        procDefMap.put(excluGateway3.getCode(), excluGateway3Node);
        procDefMap.put(dqr.getCode(), endEvent);
        /*=================== 定义网关流向变量用key ==================*/
        Map<String, String> gatewayFlowKeyMap = this.getGatewayFlowKeyMap();
        gatewayFlowKeyMap.put(excluGateway1.getCode(), "path");
        gatewayFlowKeyMap.put(excluGateway2.getCode(), "path");
        gatewayFlowKeyMap.put(excluGateway3.getCode(), "path");
    }
}
