package com.cesgroup.bdc.wfself.engine.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cesgroup.bdc.wfself.engine.mapper.*;
import com.cesgroup.bdc.wfself.engine.service.WorkflowDao;
import com.cesgroup.bdc.wfself.entity.ProcInstHisInfo;
import com.cesgroup.bdc.wfself.entity.ProcInstRunVar;
import com.cesgroup.bdc.wfself.entity.TaskRunInfo;
import com.cesgroup.bdc.wfself.entity.TaskRunPartic;
import com.cesgroup.kingkong.core.security.model.IOrganization;
import com.cesgroup.kingkong.core.security.model.IUser;
import com.cesgroup.kingkong.core.security.service.AuthsystemService;
import com.cesgroup.kingkong.shiro.ShiroUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;

/**
 *
 * @author BeiLang
 */
@Service
public class WorkFlowDaoImpl implements WorkflowDao {

    @Autowired(required = false)
    private ProcInstHisInfoMapper procInstHisInfoMapper;

    @Autowired(required = false)
    private ProcInstRunVarMapper procInstRunVarMapper;

    @Autowired(required = false)
    private TaskRunInfoMapper taskRunInfoMapper;

    @Autowired(required = false)
    private TaskRunParticMapper taskRunParticMapper;

    @Autowired
    private AuthsystemService authsystemService;

    /**
     * 根据任务信息的执行id，获取对应的历史记录信息
     *
     * @param execId
     * @return
     */
    @Override public ProcInstHisInfo queryProcInstHisInfo(String execId) {
        QueryWrapper<ProcInstHisInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("EXEC_ID", execId);
        return procInstHisInfoMapper.selectOne(queryWrapper);
    }

    /**
     * 根据的执行id 查询历史流程实例数据
     *
     * @param execIdList
     *            执行id List
     * @return
     */
    @Override public List<ProcInstHisInfo> queryProcInstHisInfoList(List<String> execIdList) {
        QueryWrapper<ProcInstHisInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("EXEC_ID", execIdList);
        return procInstHisInfoMapper.selectList(queryWrapper);
    }

    /**
     * 根据流程实例id、节点code查询，排除退回操作节点
     *
     * @param procInstId
     *            流程实例id
     * @param nodeCode
     *            节点code
     * @return
     */
    @Override public List<ProcInstHisInfo> queryProcInstHisInfoList(String procInstId, String nodeCode) {
        QueryWrapper<ProcInstHisInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("PROC_INST_ID", procInstId);
        queryWrapper.eq("NODE_CODE", nodeCode);
        queryWrapper.isNotNull("PRE_NODE_ID");
        queryWrapper.ne("PRE_NODE_ID","");
        return procInstHisInfoMapper.selectList(queryWrapper);
    }

    /**
     * 根据id获取TaskRunInfo
     *
     * @param id
     *            主键id
     * @return
     */
    @Override public TaskRunInfo queryTaskRunInfo(String id) {
        QueryWrapper<TaskRunInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("ID", id);
        return taskRunInfoMapper.selectOne(queryWrapper);
    }

    /**
     * 根据流程实例id、节点code查询
     *
     * @param procInstId
     *            流程实例id
     * @param taskCode
     *            节点code
     * @return
     */
    @Override public List<TaskRunInfo> queryTaskRunInfoList(String procInstId, String taskCode) {
        QueryWrapper<TaskRunInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("PROC_INST_ID", procInstId);
        queryWrapper.eq("TASK_CODE", taskCode);
        return taskRunInfoMapper.selectList(queryWrapper);
    }

    /**
     * 根据流程实例id、节点code查询
     *
     * @param procInstId
     *            流程实例id
     * @return
     */
    @Override public List<TaskRunInfo> queryTaskRunInfoList(String procInstId) {
        QueryWrapper<TaskRunInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("PROC_INST_ID", procInstId);
        return taskRunInfoMapper.selectList(queryWrapper);
    }

    /**
     * 根据实例id和任务code，获取参与者正序的List
     *
     * @param procInstId
     *            流程实例id
     * @param taskCode
     *            任务code
     * @return
     */
    @Override public List<TaskRunPartic> queryTaskRunParticList(String procInstId, String taskCode) {
        QueryWrapper<TaskRunPartic> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("PROC_INST_ID", procInstId);
        queryWrapper.eq("TASK_CODE", taskCode);
        queryWrapper.orderByAsc("ORDER_NUM");
        return taskRunParticMapper.selectList(queryWrapper);
    }

    /**
     * 根据实例id，查询参与人
     *
     * @param procInstId
     *            流程实例id
     * @return
     */
    @Override public List<TaskRunPartic> queryTaskRunParticList(String procInstId) {
        QueryWrapper<TaskRunPartic> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("PROC_INST_ID", procInstId);
        return taskRunParticMapper.selectList(queryWrapper);
    }

    /**
     * 查询流程实例变量
     *
     * @param procInstId
     *            流程实例id
     * @param nodeCode
     *            流程节点
     * @param name
     *            流程参数名
     * @return
     */
    @Override public ProcInstRunVar queryProcInstRunVar(String procInstId, String nodeCode, String name) {
        QueryWrapper<ProcInstRunVar> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("PROC_INST_ID", procInstId);
        queryWrapper.eq("NODE_CODE", nodeCode);
        queryWrapper.eq("NAME", name);
        return procInstRunVarMapper.selectOne(queryWrapper);
    }

    /**
     * 查询流程实例变量
     *
     * @param procInstId
     *            流程实例id
     * @param name
     *            流程参数名
     * @param data
     *            流程参数值
     * @return
     */
    @Override public List<ProcInstRunVar> queryProcInstRunVarList(String procInstId, String name, String data) {
        QueryWrapper<ProcInstRunVar> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("PROC_INST_ID", procInstId);
        queryWrapper.eq("NAME", name);
        queryWrapper.eq("DATA", data);
        return procInstRunVarMapper.selectList(queryWrapper);
    }

    /**
     * 查询流程实例变量
     *
     * @param procInstId
     *            流程实例id
     * @param nodeCode
     *            流程节点
     * @return
     */
    @Override public List<ProcInstRunVar> queryProcInstRunVarList(String procInstId, String nodeCode) {
        QueryWrapper<ProcInstRunVar> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("PROC_INST_ID", procInstId);
        queryWrapper.eq("NODE_CODE", nodeCode);
        return procInstRunVarMapper.selectList(queryWrapper);
    }

    /**
     * 查询流程实例变量
     *
     * @param procInstId
     *            流程实例id
     * @return
     */
    @Override public List<ProcInstRunVar> queryProcInstRunVarList(String procInstId) {
        QueryWrapper<ProcInstRunVar> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("PROC_INST_ID", procInstId);
        return procInstRunVarMapper.selectList(queryWrapper);
    }

    /**
     * 是否指定流程实例，未完成的只存在一个，并且执行id与之相同，或完成
     *
     * @param procInstId
     *            流程实例id
     * @param execId
     *            执行id
     * @param nodeCode
     *            流程节点
     * @return
     */
    @Override public boolean isUniqueOrCompleteUserTask(String procInstId, String execId, String nodeCode) {
        // 查询任务参与人表，如果存在数据，说明串行任务未创建
        List<TaskRunPartic> taskRunParticList = queryTaskRunParticList(procInstId, nodeCode);
        if (taskRunParticList == null || taskRunParticList.isEmpty()) {
            // 查询当前任务
            List<TaskRunInfo> taskRunInfoList = queryTaskRunInfoList(procInstId, nodeCode);
            if (taskRunInfoList != null && !taskRunInfoList.isEmpty()) {
                if (taskRunInfoList.size() == 1) {
                    TaskRunInfo taskRunInfo = taskRunInfoList.get(0);
                    // 并且未完成的任务时当前需要完成的任务
                    if (taskRunInfo.getId().equals(execId)) {
                        return true;
                    }
                }
            } else {
                return true;
            }
        }
        return false;
    }

    /**
     * 新增数据
     *
     * @param entity
     */
    @Override public void save(Object entity) {
        if (entity instanceof ProcInstHisInfo) {
            procInstHisInfoMapper.insert((ProcInstHisInfo)entity);
        }
        else if (entity instanceof ProcInstRunVar) {
            procInstRunVarMapper.insert((ProcInstRunVar)entity);
        }
        else if (entity instanceof  TaskRunInfo) {
            taskRunInfoMapper.insert((TaskRunInfo)entity);
        }
        else if (entity instanceof  TaskRunPartic) {
            taskRunParticMapper.insert((TaskRunPartic)entity);
        }
        else {
            throw new RuntimeException("com.cesgroup.bdc.wfself.engine.mapper.impl.WorkflowDaoImpl.save() 请传入正确的参数！");
        }
    }

    /**
     * 更新数据
     *
     * @param entity
     */
    @Override public void update(Object entity) {
        if (entity instanceof ProcInstHisInfo) {
            procInstHisInfoMapper.updateById((ProcInstHisInfo)entity);
        }
        else if (entity instanceof ProcInstRunVar) {
            procInstRunVarMapper.updateById((ProcInstRunVar)entity);
        }
        else if (entity instanceof  TaskRunInfo) {
            taskRunInfoMapper.updateById((TaskRunInfo)entity);
        }
        else if (entity instanceof  TaskRunPartic) {
            taskRunParticMapper.updateById((TaskRunPartic)entity);
        }
        else {
            throw new RuntimeException("com.cesgroup.bdc.wfself.engine.mapper.impl.WorkflowDaoImpl.update() 请传入正确的参数！");
        }
    }

    /**
     * 删除数据
     *
     */
    @Override public void remove(Object entity) {
        if (entity instanceof ProcInstHisInfo) {
            procInstHisInfoMapper.deleteById(((ProcInstHisInfo)entity).getId());
        }
        else if (entity instanceof ProcInstRunVar) {
            procInstRunVarMapper.deleteById(((ProcInstRunVar)entity).getId());
        }
        else if (entity instanceof  TaskRunInfo) {
            taskRunInfoMapper.deleteById(((TaskRunInfo)entity).getId());
        }
        else if (entity instanceof  TaskRunPartic) {
            taskRunParticMapper.deleteById(((TaskRunPartic)entity).getId());
        }
        else {
            throw new RuntimeException("com.cesgroup.bdc.wfself.engine.mapper.impl.WorkflowDaoImpl.remove() 请传入正确的参数！");
        }
    }

    /**
     * 删除数据
     *
     */
    @Override public void removeAll(List<?> entityList) {
        for (Object entity : entityList) {
            remove(entity);
        }
    }

    /**
     * 删除运行时数据
     *
     * @param id
     */
    @Override public void removeRunTaskInfo(String id) {
        taskRunInfoMapper.deleteById(id);
    }

    /**
     * 根据用户ID获取部门id
     *
     * @param userId
     *            用户id
     * @return orgId
     */
    @Override public String getOrgIdByUserId(String userId) {
        return getOrgByUserId(userId).getId();
    }

    /**
     * 根据用户ID获取部门对象
     *
     * @param userId
     *            用户id
     * @return Org
     */
    @Override public IOrganization getOrgByUserId(String userId) {
        return authsystemService.getOrganizationByUserId(userId);
    }

    /**
     * 根据用户ID获取用户名称
     *
     * @param userId
     *            用户id
     * @return name
     */
    @Override public String getNameById(String userId) {
        return getUserById(userId).getUserName();
    }

    /**
     * 根据用户ID获取用户对象
     *
     * @param userId
     *            用户id
     * @return User
     */
    @Override public IUser getUserById(String userId) {
        return authsystemService.getUserById(userId);
    }
}
