package com.cesgroup.kingkong.shiro.realm;

import com.cesgroup.bdc.auth.service.SystemLogService;
import com.cesgroup.bdc.logs.service.ILoginLogService;
import com.cesgroup.bdc.util.IpUtil;
import com.cesgroup.kingkong.core.ApplicationContextUtils;
import com.cesgroup.kingkong.core.security.SecurityUtils;
import com.cesgroup.kingkong.core.security.model.IOrganization;
import com.cesgroup.kingkong.core.security.model.IUser;
import com.cesgroup.kingkong.core.security.service.AuthsystemService;
import com.cesgroup.kingkong.shiro.authc.SimpleSerializableByteSource;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.authc.*;
import org.apache.shiro.authc.credential.HashedCredentialsMatcher;
import org.apache.shiro.cache.Cache;
import org.apache.shiro.cache.CacheManager;
import org.apache.shiro.web.subject.WebSubject;
import org.springframework.beans.factory.annotation.Autowired;

import javax.servlet.http.HttpServletRequest;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.atomic.AtomicInteger;
@Slf4j
public class LoginnamePasswordRealm extends UsernamePasswordRealm {
  @Autowired
  private ILoginLogService loginLogService;

  @Autowired
  private AuthsystemService authsystemService;

  @Autowired
  private SystemLogService systemLogService;
  public LoginnamePasswordRealm() {}

  //密码重试三次锁定
  // 集群中可能会导致出现验证多过5次的现象，因为AtomicInteger只能保证单节点并发
  private Cache<String, AtomicInteger> lgoinRetryCache;
  // 缓存用户的错误次数及过期时间
  private Cache<String, CacheData> loginCache;

  private int maxRetryCount = 3;

  private String lgoinRetryCacheName;

  private CacheData errorCount;

  private static SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
  //当前缓存时间
  private static String currentdate ;

  // 过期时间(单位，毫秒)
  //private long expireTime = 300000L; //5分钟
  private long expireTime = 1800000L;  //30分钟

  public void setMaxRetryCount(int maxRetryCount) {
    this.maxRetryCount = maxRetryCount;
  }

  public LoginnamePasswordRealm(CacheManager cacheManager, String lgoinRetryCacheName) {
    this.lgoinRetryCacheName = lgoinRetryCacheName;
    this.loginCache = cacheManager.getCache(lgoinRetryCacheName);
  }
  @Autowired
  private CacheManager cachemanager;

  public CacheManager getCachemanager() {
    return cachemanager;
  }

  public void setCachemanager(CacheManager cachemanager) {
    this.cachemanager = cachemanager;
  }
  //end密码重试三次锁定(李尚昆)

  @Override
  protected AuthenticationInfo doGetAuthenticationInfo(final AuthenticationToken token)
    throws AuthenticationException {
    HttpServletRequest request = (HttpServletRequest) ((WebSubject) org.apache.shiro.SecurityUtils.getSubject()).getServletRequest();
    String ip = IpUtil.getIpAddr(request);
    //获取用户
    IUser user = shiroService.loadUser(token);

    //密码重试三次锁定(李尚昆)
    String username = (String) token.getPrincipal();
    AtomicInteger tempCount = null;
    //end密码重试三次锁定(李尚昆)

    // 没找到帐号
    if (user == null) {
      UsernamePasswordToken usernamePasswordToken = (UsernamePasswordToken) token;
      loginLogService.addFail(usernamePasswordToken.getUsername(), "用户不存在", ip);
      systemLogService.addLoginLog(null, usernamePasswordToken.getUsername(), null, "失败：用户不存在", ip);
      throw new UnknownAccountException(ApplicationContextUtils.getMessage(SecurityUtils.MESSAGE_USERNAME_NOT_EXISTS));
    } else if (!user.isEnabled()) {
      loginLogService.addFail(user, "账户已锁定", ip);
      systemLogService.addLoginLog(user.getId(), user.getUserName(), null, "失败：账户已锁定", ip);
      // 帐号锁定
      throw new DisabledAccountException(ApplicationContextUtils.getMessage(SecurityUtils.MESSAGE_ACCOUNT_LOCKED));
    } else {
      //进行加盐处理
      SimpleAuthenticationInfo authenticationInfo = new SimpleAuthenticationInfo(user, user.getPassword(),
        user.getSalt() == null ? null : SimpleSerializableByteSource.bytes(user.getSalt()), this.getName());
      //密码重试三次锁定(李尚昆)
      Cache<String, CacheData> CacheDataManager = cachemanager.getCache(username);
      CacheData errorCountData  = CacheDataManager.get(username);
      //end密码重试三次锁定(李尚昆)

      try {
        //密码重试三次锁定(李尚昆)
        if(errorCountData != null&&errorCount.data.intValue() >= maxRetryCount){
          logger.error("errorCountData=>{}",errorCountData.data);
          log.info("$$$username: " + username + " $$$因用户连续输错密码次数超过{}次被锁定,请5分钟后再登录,当前登录次数：{}", maxRetryCount,errorCount.data);
          throw new ExcessiveAttemptsException("username: " + username + " tried to login more than 5 times in period");

        }else{
          UsernamePasswordToken usernamePasswordToken = (UsernamePasswordToken) token;
          this.assertCredentialsMatch(usernamePasswordToken, authenticationInfo);
          IOrganization org = authsystemService.getOrganizationByUserId(user.getId());
          loginLogService.addSuccess(user, org, "登录成功", ip);
          systemLogService.addLoginLog(user.getId(), user.getUserName(), org, "登录成功", ip);
          //清楚错误登录的缓存！！！
          loginCache = cachemanager.getCache(username);
          if(null != loginCache){
              loginCache.remove(username);
          }
          //end密码重试三次锁定(李尚昆)
        }

      } catch (AuthenticationException e) {
        loginLogService.addFail(user, "密码错误", ip);
        systemLogService.addLoginLog(user.getId(), user.getUserName(), null, "失败：密码错误", ip);
        //密码重试三次锁定(李尚昆)

        new LoginnamePasswordRealm(cachemanager,username);
        try {
          //errorCount = loginCache.get(username);
          loginCache = cachemanager.getCache(username);
          errorCount = loginCache.get(username);
          logger.error("errorCount==>{},loginCache.get(username)==>{}",errorCount,loginCache.get(username));
        } catch (NullPointerException npe) {
          logger.error("进入空指针异常！！！！");
          errorCount = null;
          //ex.printStackTrace();
        }

        if (null == errorCount) {
          tempCount = new AtomicInteger(0);
          // 记录缓存次数及过期时间
          errorCount = new CacheData(tempCount, expireTime);
          loginCache.put(username, errorCount);
          logger.error("@@@@@@errorCount==>{},loginCache.get(username)==>{}",errorCount,loginCache.get(username));
          currentdate = sf.format(new Date(Calendar.getInstance().getTimeInMillis()));
          logger.error("errorCount为空密码错误次数{}",tempCount);
        }
        if (errorCount.data.incrementAndGet() >= maxRetryCount) {
          log.info("username: " + username + " 因用户连续输错密码次数超过{}次被锁定,请5分钟后再登录,当前登录次数：{}", maxRetryCount,errorCount.data);
          /* 启动定时任务清理过期缓存，避免内存溢出 */
          Timer t = new Timer();
          /* 一分钟调度一次 */
          log.info(" **************** 开启定时清理过期的缓存任务  ，当前任务名称:{} **************** ", username);
          t.schedule(new ClearTimerTask(loginCache), 0, 60 * 1000);
          throw new ExcessiveAttemptsException("username: " + username + " tried to login more than 5 times in period");
        }
        boolean matches = new HashedCredentialsMatcher().doCredentialsMatch(token,authenticationInfo);
        if (matches) {
          // clear cache data
          loginCache.remove(username);
        }
        //e.printStackTrace();
      }
      return authenticationInfo;
    }
  }

  /**
   * 认证密码匹配调用方法
   *
   * @param token
   * @param info
   * @throws AuthenticationException
   * @author: shen.shaohua
   * @since: 2019/10/1 22:13
   */
  @Override
  protected void assertCredentialsMatch(AuthenticationToken token, AuthenticationInfo info) throws AuthenticationException {
    UsernamePasswordToken usernamePasswordToken = (UsernamePasswordToken) token;
    String password = new String(usernamePasswordToken.getPassword());
    if ("useCaUkey".equals(password) || "SSL".equals(password)) {
      System.out.println("使用Ukey登录，无需密码验证");
      return;
    }
    if ("encrypt_key".equals(password)) {
      System.out.println("使用密钥登录，无需密码验证");
      return;
    }
    logger.error("AAAAAAAAAAAAAAAAAAAAAAAAA");
    super.assertCredentialsMatch(token, info);
  }

  /**
   * 清理过期数据定时任务
   * @author WangBenYan
   */
  private static class ClearTimerTask extends TimerTask {

    Cache<String, CacheData> cache;

    public ClearTimerTask(Cache<String, CacheData> cache) {
      this.cache = cache;
    }

    @Override
    public void run() {
      Set<String> keys = cache.keys();
      log.info("ClearTimerTask run......{}", keys.toString());
      CacheData data;
      for (String key : keys) {
        data = cache.get(key);
        String overTime = sf.format(new Date(data.expireTime));
        if (data.expireTime <= 0) {
          log.info(" **************** 缓存永不过期，缓存数据:" + key + " ,错误次数:" + data.data + " ,过期时间:" + overTime+ " ,缓存时间: " + currentdate);
          continue;
        }
        /* 到期时间 > 当前时间，则继续 */
        if (data.expireTime > Calendar.getInstance().getTimeInMillis()) {
          log.info(" **************** 缓存的用户 :" + key + " ,错误次数:" + data.data + " ,过期时间:" + overTime+ " ,缓存时间: " + currentdate);
          continue;
        }
        log.info(" **************** 开始清除缓存 **************** : " + key);
        cache.remove(key);
        log.info(" **************** 定时清理过期的缓存... {} ", cache.get(key));
        this.cancel();
      }
    }

  }
  private static class CacheData {
    // 缓存数据
    private AtomicInteger data;
    // 过期时间(单位，毫秒)
    private long expireTime;
    public CacheData(){};
    public CacheData(AtomicInteger errorcount, long expire) {
      log.info(" **************** 缓存认证错误次数:" + errorcount + " 过期时间毫秒数:" + expire);
      this.data = errorcount;
      if (expire <= 0) {
        this.expireTime = 0L;
      } else {
        this.expireTime = Calendar.getInstance().getTimeInMillis() + expire;
      }
    }
  }
}
