/**
 * <p>上海中信信息发展股份有限公司 Copyright(c) 2019</p>
 * <p>http://www.cesgroup.com.cn/</p>
 * <p>2019-5-13 21:25 by 大数据创建中心</p>
 */

package com.cesgroup.kingkong.shiro.service;

import com.cesgroup.kingkong.core.security.model.IUser;
import com.cesgroup.kingkong.core.security.service.AuthenticateService;
import org.apache.shiro.authc.AuthenticationToken;
import org.apache.shiro.authc.UsernamePasswordToken;

import java.util.Set;

/**
 * 用户认证实现
 *
 * @author 数据创建中心
 * @version 1.0.0 2019-5-13
 */
public class PlatformAuthenticateService implements ShiroAuthenticateService {

  private AuthenticateService authenticateService;

  /**
   * @return the authenticateService
   */
  public AuthenticateService getAuthenticateService() {
    return authenticateService;
  }

  /**
   * @param authenticateService the authenticateService to set
   */
  public void setAuthenticateService(final AuthenticateService authenticateService) {
    this.authenticateService = authenticateService;
  }

  /**
   * @see ShiroAuthenticateService
   * #loadUser(org.apache.shiro.authc.AuthenticationToken)
   */
  @Override
  public IUser loadUser(final AuthenticationToken token) {
    UsernamePasswordToken usernamePasswordToken = (UsernamePasswordToken) token;
    String loginName = usernamePasswordToken.getUsername();
    return getAuthenticateService().loadUser(loginName);
  }

  /**
   * @see ShiroAuthenticateService#isTenant()
   */
  @Override
  public boolean isTenant() {
    return false;
  }

  /**
   * 获取角色信息
   *
   * @param user 用户信息
   * @return 角色信息集合
   * @author: shen.shaohua
   * @since: 2020/4/1 10:32
   */
  public Set<String> loadRoles(IUser user) {
    Set<String> roles = getAuthenticateService().loadRoles(user);
    return roles;
  }

  /**
   * 获取权限信息
   *
   * @param user 用户信息
   * @return 权限信息集合
   * @author: shen.shaohua
   * @since: 2020/4/1 10:32
   */
  public Set<String> loadPermissions(IUser user) {
    Set<String> permissions = getAuthenticateService().loadPermissions(user);
    return permissions;
  }
}
